﻿package ;

import com.georg.Mandelbrot;
import com.georg.Stepper;
import com.georg.tools.FPSCounter;
import flash.display.Sprite;
import flash.display.Stage;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.events.TimerEvent;
import flash.Lib;
import flash.utils.Timer;

/**
 * ...
 * @author Georg Graf
 */

class Main extends Sprite
{
	public static var stage:Stage;
	private static var mb:Mandelbrot;
	
	private static var scale:Float;
	private static var colorFactor:Float;
	
	private static var stepper:Stepper;
	
	private static var fpscounter:FPSCounter;
	
	// dragging vars
	private static var dragTimer:Timer;
	private static var dragging:Bool;
	private static var dragPrevX:Int;
	private static var dragPrevY:Int;
	
	static function main() 
	{
		// init stage
		stage = Lib.current.stage;
		stage.addEventListener(MouseEvent.MOUSE_DOWN, stageDownHandler);
		stage.addEventListener(MouseEvent.MOUSE_UP, stageUpHandler);
		stage.addEventListener(MouseEvent.MOUSE_WHEEL, stageWheelListener);
		
		// create Mandelbrot
		mb = new Mandelbrot(stage.stageWidth, stage.stageHeight);
		Lib.current.addChild(mb);
		
		// create Stepper
		stepper = new Stepper(20);
		stepper.addEventListener(Event.CHANGE, draw);
		stepper.x = 5;
		stepper.y = 5;
		Lib.current.addChild(stepper);
		
		// create fps counter
		fpscounter = new FPSCounter();
		fpscounter.x = stage.stageWidth - 90;
		fpscounter.y = 10;
		Lib.current.addChild(fpscounter);
		
		// create one picture
		scale = .01;
		colorFactor = 0;
		stageClick(false);
		
		// create drag timer
		dragging = false;
		dragTimer = new Timer(200, 1);
		dragTimer.addEventListener(TimerEvent.TIMER_COMPLETE, setDrag);
	}
	
	static private function stageWheelListener(e:MouseEvent):Void 
	{
		scale -= scale * (.05 * e.delta);
		stageClick();
	}
	
	static private function setDrag(e:TimerEvent):Void 
	{
		mb.stop();
		dragPrevX = cast(stage.mouseX, Int);
		dragPrevY = cast(stage.mouseY, Int);
		dragging = true;
		stage.addEventListener(MouseEvent.MOUSE_MOVE, drag);
	}
	
	static private function stageUpHandler(e:MouseEvent = null):Void 
	{
		dragTimer.stop();
		
		if (e.target.name == null)
		{
			if (dragging) // DRAGGING END
			{
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, drag);
				dragging = false;
				stageClick();
			}
			else // CLICK
			{
				stageClick();
			}
		}
	}
	static private function stageDownHandler(e:MouseEvent = null):Void 
	{
		if (e.target.name == null)
		{
			dragTimer.reset();
			dragTimer.start();
		}
	}
	
	static private function drag(e:MouseEvent):Void 
	{
		mb.drag(cast(stage.mouseX, Int) - dragPrevX, cast(stage.mouseY, Int) - dragPrevY);
				
		dragPrevX = cast(stage.mouseX, Int);
		dragPrevY = cast(stage.mouseY, Int);
	}
	
	static private function stageClick(setOffset:Bool = true):Void
	{
		if (setOffset)
		{
			//mb.offsetX = stage.mouseX - (stage.stageWidth / 2);
			//mb.offsetY = stage.mouseY - (stage.stageHeight / 2);
		}
		draw();
		
		//scale -= scale * .05;
		colorFactor += .05;
		if (colorFactor > 1) colorFactor = 0;
	}
	
	static private function draw(e:Event = null):Void
	{
		// start frame counter
		fpscounter.start();
		
		//mb.drawIncremental(10, 0, stepper.value, scale, cast(colorFactor * 360), drawComplete);
		mb.drawPixelIncremental(100, 0, stepper.value, scale, cast(colorFactor * 360), drawComplete);
	}
	
	static private function drawComplete():Void
	{
		// stop frame counter
		fpscounter.stop();
	}
	
}